package javawrapper;

/**
 * This holds all data relating to allometry.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 25, 2005: Created (LEM)
 */

public class Allometry
    extends WorkerBase {

  /**Species-specific - maximum crown height.*/
  protected ModelVector mp_fMaxCanopyHeight = new ModelVector(
      "Maximum Tree Height, in meters", "tr_canopyHeight", "tr_chVal", 0,
      ModelVector.FLOAT, true);

  /**Species-specific - slope of asymptotic height.*/
  protected ModelVector mp_fSlopeOfAsymptoticHeight = new ModelVector(
      "Slope of Asymptotic Height", "tr_slopeOfAsymHeight", "tr_soahVal",
      0, ModelVector.FLOAT, true);

  /**Species-specific - exponent in crown radius equation.*/
  protected ModelVector mp_fCrownRadExp = new ModelVector(
      "Crown Radius Exponent", "tr_stdCrownRadExp", "tr_screVal", 0,
      ModelVector.FLOAT, true);

  /**Species-specific - exponent in crown height equation.*/
  protected ModelVector mp_fCrownHtExp = new ModelVector(
      "Crown Height Exponent", "tr_stdCrownHtExp", "tr_scheVal", 0,
      ModelVector.FLOAT, true);

  /**Species-specific - slope of asymptotic crown radius*/
  protected ModelVector mp_fSlopeOfAsympCrownRad = new ModelVector(
      "Slope of Asymptotic Crown Radius", "tr_stdAsympCrownRad", "tr_sacrVal",
      0, ModelVector.FLOAT, true);

  /**Species-specific - slope of the asymptotic crown height*/
  protected ModelVector mp_fSlopeOfAsympCrownHt = new ModelVector(
      "Slope of Asymptotic Crown Height", "tr_stdAsympCrownHt",
      "tr_sachVal", 0, ModelVector.FLOAT, true);

  /**Species-specific - slope of the height-diam10 relationship.*/
  protected ModelVector mp_fSlopeOfHeightDiam10 = new ModelVector(
      "Slope of Height-Diameter at 10 cm Relationship",
      "tr_slopeOfHeight-Diam10", "tr_sohdVal", 0, ModelVector.FLOAT, true);

  /**Species-specific - slope of relationship of diam10 to dbh.*/
  protected ModelVector mp_fDiam10ToDbhSlope = new ModelVector(
      "Slope of DBH to Diameter at 10 cm Relationship", "tr_conversionDiam10ToDBH",
      "tr_cdtdVal", 0, ModelVector.FLOAT, true);

  /**Species-specific - intercept of relationship of diam10 to dbh.*/
  protected ModelVector mp_fDiam10ToDbhIntercept = new ModelVector(
      "Intercept of DBH to Diameter at 10 cm Relationship",
      "tr_interceptDiam10ToDBH", "tr_idtdVal", 0, ModelVector.FLOAT, true);

  /**Slope of linear height-DBH relationship for adults - not
          required if the linear relationship isn't used*/
  protected ModelVector mp_fAdultLinearSlope = new ModelVector(
      "Adult Linear Function Slope", "tr_adultLinearSlope",
      "tr_alsVal", 0, ModelVector.FLOAT, true);

  /**Intercept of linear height-DBH relationship for adults - not
          required if the linear relationship isn't used*/
  protected ModelVector mp_fAdultLinearIntercept = new ModelVector(
      "Adult Linear Function Intercept", "tr_adultLinearIntercept",
      "tr_aliVal", 0, ModelVector.FLOAT, true);

  /**Slope of linear height-DBH relationship for saplings - not
          required if the linear relationship isn't used*/
  protected ModelVector mp_fSaplingLinearSlope = new ModelVector(
      "Sapling Linear Function Slope", "tr_saplingLinearSlope",
      "tr_salsVal", 0, ModelVector.FLOAT, true);

  /**Intercept of linear height-DBH relationship for saplings - not
          required if the linear relationship isn't used*/
  protected ModelVector mp_fSaplingLinearIntercept = new ModelVector(
      "Sapling Linear Function Intercept", "tr_saplingLinearIntercept",
      "tr_aliVal", 0, ModelVector.FLOAT, true);

  /**Slope of linear height-DBH relationship for seedlings - not
          required if the linear relationship isn't used*/
  protected ModelVector mp_fSeedlingLinearSlope = new ModelVector(
      "Seedling Linear Function Slope", "tr_seedlingLinearSlope",
      "tr_selsVal", 0, ModelVector.FLOAT, true);

  /**Intercept of linear height-DBH relationship for seedlings - not
          required if the linear relationship isn't used*/
  protected ModelVector mp_fSeedlingLinearIntercept = new ModelVector(
      "Seedling Linear Function Intercept", "tr_seedlingLinearIntercept",
      "tr_seliVal", 0, ModelVector.FLOAT, true);

  /**Slope of reverse linear height-DBH relationship for adults - not
          required if the reverse linear relationship isn't used*/
  protected ModelVector mp_fAdultReverseLinearSlope = new ModelVector(
      "Adult Reverse Linear Function Slope", "tr_adultReverseLinearSlope",
      "tr_arlsVal", 0, ModelVector.FLOAT, true);

  /**Intercept of reverse linear height-DBH relationship for adults - not
          required if the reverse linear relationship isn't used*/
  protected ModelVector mp_fAdultReverseLinearIntercept = new ModelVector(
      "Adult Reverse Linear Function Intercept",
      "tr_adultReverseLinearIntercept", "tr_arliVal", 0, ModelVector.FLOAT, true);

  /**Slope of reverse linear height-DBH relationship for saplings - not
          required if the reverse linear relationship isn't used*/
  protected ModelVector mp_fSaplingReverseLinearSlope = new ModelVector(
      "Sapling Reverse Linear Function Slope", "tr_saplingReverseLinearSlope",
      "tr_sarlsVal", 0, ModelVector.FLOAT, true);

  /**Intercept of reverse linear height-DBH relationship for saplings - not
          required if the reverse linear relationship isn't used*/
  protected ModelVector mp_fSaplingReverseLinearIntercept = new ModelVector(
      "Sapling Reverse Linear Function Intercept",
      "tr_saplingReverseLinearIntercept", "tr_sarliVal", 0, ModelVector.FLOAT, true);

  /**Slope of reverse linear height-DBH relationship for seedlings - not
          required if the reverse linear relationship isn't used*/
  protected ModelVector mp_fSeedlingReverseLinearSlope = new ModelVector(
      "Seedling Reverse Linear Function Slope", "tr_seedlingReverseLinearSlope",
      "tr_serlsVal", 0, ModelVector.FLOAT, true);

  /**Intercept of reverse linear height-DBH relationship for seedlings - not
          required if the reverse linear relationship isn't used*/
  protected ModelVector mp_fSeedlingReverseLinearIntercept = new ModelVector(
      "Seedling Reverse Linear Function Intercept",
      "tr_seedlingReverseLinearIntercept", "tr_serliVal", 0, ModelVector.FLOAT, true);

  /**Which relationship to use to relate height to diam10 for
          seedlings - 0 = standard, 1 = linear, 2 = reverse linear*/
  protected ModelVector mp_iWhatSeedlingHDFunction = new ModelVector(
      "Seedling Height-Diameter Function",
      "tr_whatSeedlingHeightDiam", "tr_wsehdVal", 0, ModelVector.MODEL_ENUM, true);

  /**Which relationship to use to relate height to DBH for
          saplings - 0 = standard, 1 = linear, 2 = reverse linear*/
  protected ModelVector mp_iWhatSaplingHDFunction = new ModelVector(
      "Sapling Height-Diameter Function",
      "tr_whatSaplingHeightDiam", "tr_wsahdVal", 0, ModelVector.MODEL_ENUM, true);

  /**Which relationship to use to relate height to DBH for
          adults - 0 = standard, 1 = linear, 2 = reverse linear*/
  protected ModelVector mp_iWhatAdultHDFunction = new ModelVector(
      "Adult Height-Diameter Function",
      "tr_whatAdultHeightDiam", "tr_wahdVal", 0, ModelVector.MODEL_ENUM, true);

  /**Which relationship to use to relate DBH to crown radius for
          adults - 0 = standard, 1 = Chapman-Richards*/
  protected ModelVector mp_iWhatAdultCRDFunction = new ModelVector(
      "Adult Crown Radius-Diameter Function",
      "tr_whatAdultCrownRadDiam", "tr_wacrdVal", 0, ModelVector.MODEL_ENUM, true);

  /**Which relationship to use to relate DBH to crown radius for
          saplings - 0 = standard, 1 = Chapman-Richards*/
  protected ModelVector mp_iWhatSaplingCRDFunction = new ModelVector(
      "Sapling Crown Radius-Diameter Function",
      "tr_whatSaplingCrownRadDiam", "tr_wscrdVal", 0, ModelVector.MODEL_ENUM, true);

  /**Which relationship to use to relate height to crown height for
          adults - 0 = standard, 1 = Chapman-Richards*/
  protected ModelVector mp_iWhatAdultCHHFunction = new ModelVector(
      "Adult Crown Height-Height Function",
      "tr_whatAdultCrownHeightHeight", "tr_wachhVal", 0, ModelVector.MODEL_ENUM, true);

  /**Which relationship to use to relate height to crown height for
          saplings - 0 = standard, 1 = Chapman-Richards*/
  protected ModelVector mp_iWhatSaplingCHHFunction = new ModelVector(
      "Sapling Crown Height-Height Function",
      "tr_whatSaplingCrownHeightHeight", "tr_wschhVal", 0,
      ModelVector.MODEL_ENUM, true);

  /**Intercept of Chapman-Richards crown radius-DBH relationship - not
   * required if the Chapman-Richards crown radius relationship isn't used*/
  protected ModelVector mp_fCRCrownRadIntercept = new ModelVector(
      "Chapman-Richards Crown Radius Intercept",
      "tr_chRichCrownRadIntercept", "tr_crcriVal", 0, ModelVector.FLOAT, true);

  /**Asymptotic crown radius of Chapman-Richards crown radius-DBH
   * relationship - not required if the Chapman-Richards crown radius
   * relationship isn't used*/
  protected ModelVector mp_fCRAsympCrownRad = new ModelVector(
      "Chapman-Richards Asymptotic Crown Radius",
      "tr_chRichCrownRadAsymp", "tr_crcraVal", 0, ModelVector.FLOAT, true);

  /**Shape parameter 1 (b) of Chapman-Richards crown radius-DBH relationship
   * - not required if the C-R crown radius relationship isn't used*/
  protected ModelVector mp_fCRCrownRadShape1 = new ModelVector(
      "Chapman-Richards Crown Radius Shape 1 (b)",
      "tr_chRichCrownRadShape1b", "tr_crcrs1bVal", 0, ModelVector.FLOAT, true);

  /**Shape parameter 2 (c) of Chapman-Richards crown radius-DBH relationship
   * - not required if the C-R crown radius relationship isn't used*/
  protected ModelVector mp_fCRCrownRadShape2 = new ModelVector(
      "Chapman-Richards Crown Radius Shape 2 (c)",
      "tr_chRichCrownRadShape2c", "tr_crcrs2cVal", 0, ModelVector.FLOAT, true);

  /**Intercept of Chapman-Richards crown height-height relationship - not
   * required if the Chapman-Richards crown height relationship isn't used*/
  protected ModelVector mp_fCRCrownHtIntercept = new ModelVector(
      "Chapman-Richards Crown Height Intercept",
      "tr_chRichCrownHtIntercept", "tr_crchiVal", 0, ModelVector.FLOAT, true);

  /**Asymptotic crown radius of Chapman-Richards crown height-height
   * relationship - not required if the Chapman-Richards crown height
   * relationship isn't used*/
  protected ModelVector mp_fCRAsympCrownHt = new ModelVector(
      "Chapman-Richards Asymptotic Crown Height",
      "tr_chRichCrownHtAsymp", "tr_crchaVal", 0, ModelVector.FLOAT, true);

  /**Shape parameter 1 (b) of Chapman-Richards crown height-height relationship
   * - not required if the C-R crown height relationship isn't used*/
  protected ModelVector mp_fCRCrownHtShape1 = new ModelVector(
      "Chapman-Richards Crown Height Shape 1 (b)",
      "tr_chRichCrownHtShape1b", "tr_crchs1bVal", 0, ModelVector.FLOAT, true);

  /**Shape parameter 2 (c) of Chapman-Richards crown height-height relationship
   * - not required if the C-R crown height relationship isn't used*/
  protected ModelVector mp_fCRCrownHtShape2 = new ModelVector(
      "Chapman-Richards Crown Height Shape 2 (c)",
      "tr_chRichCrownHtShape2c", "tr_crchs2cVal", 0, ModelVector.FLOAT, true);

  /** "a" parameter of power function height-diam relationship - not required
   * if this function isn't used (LEM 1-25-05)*/
  protected ModelVector mp_fPowerA = new ModelVector(
      "Power Function \"a\"", "tr_saplingPowerA", "tr_sapaVal", 0, ModelVector.FLOAT, true);

  /** "b" parameter of power function height-diam relationship - not required
   * if this function isn't used (LEM 1-25-05)*/
  protected ModelVector mp_fPowerB = new ModelVector(
      "Power Function Exponent \"b\"", "tr_saplingPowerB", "tr_sapbVal", 0, ModelVector.FLOAT, true);


  /**
   * Constructor
   * @param oManager GUIManager object
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 25, 2005: Created (LEM)
   */
  public Allometry(GUIManager oManager) {
    super(oManager, "Allometry", "allometry");

    m_sHelpID = "data.allometry";

    mp_oAllData.add(mp_iWhatAdultHDFunction);
    mp_oAllData.add(mp_iWhatSaplingHDFunction);
    mp_oAllData.add(mp_iWhatSeedlingHDFunction);
    mp_oAllData.add(mp_iWhatAdultCRDFunction);
    mp_oAllData.add(mp_iWhatSaplingCRDFunction);
    mp_oAllData.add(mp_iWhatAdultCHHFunction);
    mp_oAllData.add(mp_iWhatSaplingCHHFunction);
    mp_oAllData.add(mp_fMaxCanopyHeight);
    mp_oAllData.add(mp_fSlopeOfAsympCrownRad);
    mp_oAllData.add(mp_fCrownRadExp);
    mp_oAllData.add(mp_fSlopeOfAsympCrownHt);
    mp_oAllData.add(mp_fCrownHtExp);
    mp_oAllData.add(mp_fDiam10ToDbhSlope);
    mp_oAllData.add(mp_fDiam10ToDbhIntercept);
    mp_oAllData.add(mp_fSlopeOfAsymptoticHeight);
    mp_oAllData.add(mp_fSlopeOfHeightDiam10);
    mp_oAllData.add(mp_fAdultLinearSlope);
    mp_oAllData.add(mp_fAdultLinearIntercept);
    mp_oAllData.add(mp_fAdultReverseLinearSlope);
    mp_oAllData.add(mp_fAdultReverseLinearIntercept);
    mp_oAllData.add(mp_fSaplingLinearSlope);
    mp_oAllData.add(mp_fSaplingLinearIntercept);
    mp_oAllData.add(mp_fSaplingReverseLinearSlope);
    mp_oAllData.add(mp_fSaplingReverseLinearIntercept);
    mp_oAllData.add(mp_fPowerA);
    mp_oAllData.add(mp_fPowerB);
    mp_oAllData.add(mp_fSeedlingLinearSlope);
    mp_oAllData.add(mp_fSeedlingLinearIntercept);
    mp_oAllData.add(mp_fSeedlingReverseLinearSlope);
    mp_oAllData.add(mp_fSeedlingReverseLinearIntercept);
    mp_oAllData.add(mp_fCRCrownRadIntercept);
    mp_oAllData.add(mp_fCRAsympCrownRad);
    mp_oAllData.add(mp_fCRCrownRadShape1);
    mp_oAllData.add(mp_fCRCrownRadShape2);
    mp_oAllData.add(mp_fCRCrownHtIntercept);
    mp_oAllData.add(mp_fCRAsympCrownHt);
    mp_oAllData.add(mp_fCRCrownHtShape1);
    mp_oAllData.add(mp_fCRCrownHtShape2);
  }

  /**
   * Does setup.  Sets up the enums for the function lists.
   * @param oPop TreePopulation object.
   * @throws ModelException passing through from called functions.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 25, 2005: Created (LEM)
   */
  public void DoSetup(TreePopulation oPop) throws ModelException {
    ModelEnum oEnum;
    int iNumSpecies = oPop.GetNumberOfSpecies(), i;

    //Set up the "which function" vector and default all functions to standard
    mp_iWhatAdultHDFunction.GetValue().clear();
    mp_iWhatSaplingHDFunction.GetValue().clear();
    mp_iWhatSeedlingHDFunction.GetValue().clear();
    mp_iWhatAdultCRDFunction.GetValue().clear();
    mp_iWhatSaplingCRDFunction.GetValue().clear();
    mp_iWhatAdultCHHFunction.GetValue().clear();
    mp_iWhatSaplingCHHFunction.GetValue().clear();
    for (i = 0; i < iNumSpecies; i++) {
      oEnum = new ModelEnum(new int[] {
                            2, 1, 0}
                            , new String[] {"Reverse Linear",
                            "Linear", "Standard"}
                            , "Function used", "");
      oEnum.SetValue("Standard");
      mp_iWhatAdultHDFunction.GetValue().add(oEnum);

      oEnum = new ModelEnum(new int[] {
                            3, 2, 1, 0}
                            , new String[] {"Power", "Reverse Linear",
                            "Linear", "Standard"},
                            "Function used", "");
      oEnum.SetValue("Standard");
      mp_iWhatSaplingHDFunction.GetValue().add(oEnum);

      oEnum = new ModelEnum(new int[] {2, 1, 0}
                            , new String[] {"Reverse Linear",
                            "Linear", "Standard"},
                            "Function used", "");
      oEnum.SetValue("Standard");
      mp_iWhatSeedlingHDFunction.GetValue().add(oEnum);

      oEnum = new ModelEnum(new int[] {1, 0}
                            , new String[] {"Chapman-Richards", "Standard"},
                            "Function used", "");
      oEnum.SetValue("Standard");
      mp_iWhatAdultCRDFunction.GetValue().add(oEnum);

      oEnum = new ModelEnum(new int[] {1, 0}
                            , new String[] {"Chapman-Richards", "Standard"},
                            "Function used", "");
      oEnum.SetValue("Standard");
      mp_iWhatSaplingCRDFunction.GetValue().add(oEnum);

      oEnum = new ModelEnum(new int[] {1, 0}
                            , new String[] {"Chapman-Richards", "Standard"},
                            "Function used", "");
      oEnum.SetValue("Standard");
      mp_iWhatAdultCHHFunction.GetValue().add(oEnum);

      oEnum = new ModelEnum(new int[] {1, 0}
                            , new String[] {"Chapman-Richards", "Standard"},
                            "Function used", "");
      oEnum.SetValue("Standard");
      mp_iWhatSaplingCHHFunction.GetValue().add(oEnum);
    }
    //Default diam10 to DBH intercept to 0
    if (mp_fDiam10ToDbhIntercept.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fDiam10ToDbhIntercept.GetValue().add(new Float(0.0));
      }
    }
  }

  /**
   * This makes sure all data is valid and can be used to run the model.
   * @throws ModelException in any of the following cases:
   * <ul>
   * <li> There is any value less than or equal to zero in any of the following
   * vectors, when used:
   * <ul>
   * <li>mp_fSlopeOfAsympCrownRad</li>
   * <li>mp_fDiam10ToDbhConversion</li>
   * <li>mp_fSlopeOfAsympCrownHt</li>
   * <li>mp_fMaxCanopyHeight</li>
   * <li>mp_fSlopeOfAsymptoticHeight</li>
   * </ul></li>
   * <li>A value in any slope is 0 (if it is being used)</li>
   * </ul>
   * @param oPop Not used.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    int iNumSpecies = oPop.GetNumberOfSpecies(), i;
    boolean[] p_bApplies = new boolean[iNumSpecies];

    //Validate the size of the species-specific vectors
    MakeSureRightSize(mp_fCrownRadExp, iNumSpecies);
    MakeSureRightSize(mp_fSlopeOfAsympCrownRad, iNumSpecies);
    MakeSureRightSize(mp_fDiam10ToDbhSlope, iNumSpecies);
    MakeSureRightSize(mp_fDiam10ToDbhIntercept, iNumSpecies);
    MakeSureRightSize(mp_fSlopeOfAsympCrownHt, iNumSpecies);
    MakeSureRightSize(mp_fCrownHtExp, iNumSpecies);
    MakeSureRightSize(mp_fMaxCanopyHeight, iNumSpecies);

    //Slope of asymptotic crown height should not be negative
    for (i = 0; i < p_bApplies.length; i++) {
      if (1 == ((ModelEnum)mp_iWhatAdultCHHFunction.GetValue().elementAt(i)).GetValue() &&
          1 == ((ModelEnum)mp_iWhatSaplingCHHFunction.GetValue().elementAt(i)).GetValue())
        p_bApplies[i] = false;
      else p_bApplies[i] = true;
    }
    MakeSureAllPositive(mp_fSlopeOfAsympCrownHt, p_bApplies);

    MakeSureAllPositive(mp_fMaxCanopyHeight);

    //Slope of asymptotic crown radius should not be negative
    for (i = 0; i < p_bApplies.length; i++) {
      if (1 == ((ModelEnum)mp_iWhatAdultCRDFunction.GetValue().elementAt(i)).GetValue() &&
          1 == ((ModelEnum)mp_iWhatSaplingCRDFunction.GetValue().elementAt(i)).GetValue())
        p_bApplies[i] = false;
      else p_bApplies[i] = true;
    }
    MakeSureAllPositive(mp_fSlopeOfAsympCrownRad, p_bApplies);

    MakeSureAllPositive(mp_fDiam10ToDbhSlope);
    MakeSureAllPositive(mp_fMaxCanopyHeight);

    //Make sure the values for asymptotic height that are used are positive
    for (i = 0; i < p_bApplies.length; i++) {
      p_bApplies[i] = false;
    }
    for (i = 0; i < iNumSpecies; i++) {
      if ( ( (ModelEnum) mp_iWhatAdultHDFunction.GetValue().elementAt(i)).
          GetValue() == 0 ||
          ( (ModelEnum) mp_iWhatSaplingHDFunction.GetValue().elementAt(i)).
          GetValue() == 0) {
        p_bApplies[i] = true;
      }
    }
    MakeSureAllPositive(mp_fSlopeOfAsymptoticHeight, p_bApplies);

    //Make sure adult slopes that are used are non-zero
    for (i = 0; i < p_bApplies.length; i++) {
      p_bApplies[i] = false;
    }
    for (i = 0; i < iNumSpecies; i++) {
      if ( ( (ModelEnum) mp_iWhatAdultHDFunction.GetValue().elementAt(i)).
          GetValue() == 1) {
        p_bApplies[i] = true;
      }
    }
    MakeSureAllNonZero(mp_fAdultLinearSlope, p_bApplies);

    for (i = 0; i < p_bApplies.length; i++) {
      p_bApplies[i] = false;
    }
    for (i = 0; i < iNumSpecies; i++) {
      if ( ( (ModelEnum) mp_iWhatAdultHDFunction.GetValue().elementAt(i)).
          GetValue() == 2) {
        p_bApplies[i] = true;
      }
    }
    MakeSureAllNonZero(mp_fAdultReverseLinearSlope, p_bApplies);

    //Make sure sapling slopes that are used are non-zero
    for (i = 0; i < p_bApplies.length; i++) {
      p_bApplies[i] = false;
    }
    for (i = 0; i < iNumSpecies; i++) {
      if ( ( (ModelEnum) mp_iWhatSaplingHDFunction.GetValue().elementAt(i)).
          GetValue() == 1) {
        p_bApplies[i] = true;
      }
    }
    MakeSureAllNonZero(mp_fSaplingLinearSlope, p_bApplies);

    for (i = 0; i < p_bApplies.length; i++) {
      p_bApplies[i] = false;
    }
    for (i = 0; i < iNumSpecies; i++) {
      if ( ( (ModelEnum) mp_iWhatSaplingHDFunction.GetValue().elementAt(i)).
          GetValue() == 2) {
        p_bApplies[i] = true;
      }
    }
    MakeSureAllNonZero(mp_fSaplingReverseLinearSlope, p_bApplies);

    //Make sure seedling slopes that are used are non-zero
    for (i = 0; i < p_bApplies.length; i++) {
      p_bApplies[i] = false;
    }
    for (i = 0; i < iNumSpecies; i++) {
      if ( ( (ModelEnum) mp_iWhatSeedlingHDFunction.GetValue().elementAt(i)).
          GetValue() == 1) {
        p_bApplies[i] = true;
      }
    }
    MakeSureAllNonZero(mp_fSeedlingLinearSlope, p_bApplies);

    for (i = 0; i < p_bApplies.length; i++) {
      p_bApplies[i] = false;
    }
    for (i = 0; i < iNumSpecies; i++) {
      if ( ( (ModelEnum) mp_iWhatSeedlingHDFunction.GetValue().elementAt(i)).
          GetValue() == 2) {
        p_bApplies[i] = true;
      }
    }
    MakeSureAllNonZero(mp_fSeedlingReverseLinearSlope, p_bApplies);
  }

  /**
   * Overridden for backwards compatibility after changing XML tag names.  This
   * was done in version 6.05 and can be removed after several versions.
   * @param sXMLTag Parent XML tag of data vector whose value is to be set.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param p_oData Vector of data values appropriate to the data type
   * @param p_sChildXMLTags The XML tags of the child elements
   * @param p_bAppliesTo Array of booleans saying which of the vector values
   * should be set.  This is important in the case of species-specifics - the
   * vector index is the species number but not all species are set.
   * @param oParentAttributes Attributes of parent tag.  May be useful when
   * overridding this for unusual tags.
   * @param p_oAttributes Attributes passed from parser.  This may be needed
   * when overriding this function.  Basic species-specific values are
   * already handled by this function.
   * @return true if the value was set successfully; false if the value could
   * not be found.  (This would not be an error, because I need a way to
   * cycle through the objects until one of the objects comes up with a
   * match.)  If a match to a data object is made via XML tag, but the found
   * object is not a ModelVector, this returns false.
   * @throws ModelException if the value could not be assigned to the data
   * object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 7, 2005: Created (LEM)
   */
  public boolean SetVectorValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        java.util.Vector p_oData,
                                        String[] p_sChildXMLTags,
                                        boolean[] p_bAppliesTo,
                                        org.xml.sax.Attributes
                                        oParentAttributes,
                                        org.xml.sax.Attributes[] p_oAttributes) throws
      ModelException {

      int i;
      if (sXMLTag.equals("tr_canopyRadiusExponent")) {
        sXMLTag = "tr_stdCrownRadExp";
        for (i = 0; i < p_sChildXMLTags.length; i++) {
          if (p_sChildXMLTags[i].equals("tr_creVal")) {
            p_sChildXMLTags[i] = "tr_screVal";
          }
        }
      }
      else if (sXMLTag.equals("tr_canopyDepthExponent")) {
        sXMLTag = "tr_stdCrownHtExp";
        for (i = 0; i < p_sChildXMLTags.length; i++) {
          if (p_sChildXMLTags[i].equals("tr_cdeVal")) {
            p_sChildXMLTags[i] = "tr_scheVal";
          }
        }
      }
      else if (sXMLTag.equals("tr_canopyRadiusToDBH")) {
        sXMLTag = "tr_stdAsympCrownRad";
        for (i = 0; i < p_sChildXMLTags.length; i++) {
          if (p_sChildXMLTags[i].equals("tr_crtdVal")) {
            p_sChildXMLTags[i] = "tr_sacrVal";
          }
        }
      }
      else if (sXMLTag.equals("tr_fractOfHeightInCanopy")) {
        sXMLTag = "tr_stdAsympCrownHt";
        for (i = 0; i < p_sChildXMLTags.length; i++) {
          if (p_sChildXMLTags[i].equals("tr_fohicVal")) {
            p_sChildXMLTags[i] = "tr_sachVal";
          }
        }
      }
    return super.SetVectorValueByXMLTag(sXMLTag, sXMLParentTag, p_oData,
                                        p_sChildXMLTags, p_bAppliesTo,
                                        oParentAttributes, p_oAttributes);
  }
}
