package javawrapper;

/**
 * This is a base class for packaging data values with additional information.
 * The additional information allows for automation of lots of data operations.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public abstract class ModelData {
  /**Descriptive string.  This should be meaningful to the user because it
   * will be used in error messages and entry windows.*/
  protected String m_sDescriptor,
      /**XML tag*/
      m_sXMLTag;

  /**
   * Constructor
   * @param sDescriptor Descriptor string.
   * @param sXMLTag The XML tag associated with this piece of data.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public ModelData(String sDescriptor, String sXMLTag) {
    m_sDescriptor = sDescriptor;
    m_sXMLTag = sXMLTag;
  }

  /**
   * Returns the descriptor string.
   * @return The Descriptor string.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String GetDescriptor() {
    return m_sDescriptor;
  }

  /**
   * Returns the XML tag associated with this piece of data.
   * @return The XML tag.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String GetXMLTag() {
    return m_sXMLTag;
  }

  /**
   * String representation of the value.  Required to be overridden.
   * @return String representation of the value.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public abstract String toString();

}