package javawrapper;

import junit.framework.*;
import java.util.*;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.InputSource;

/**
 * Tests the Harvest and Natural Disturbance behaviors of the
* DisturbanceBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)

 */
public class TestHarvestBehaviors
    extends TestCase {
  private DisturbanceBehaviors m_oHarvestBehaviors = null;

  protected void setUp() throws Exception {
    super.setUp();
    GUIManager oManager = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      m_oHarvestBehaviors = new DisturbanceBehaviors(oManager);
    } catch (ModelException oErr) {
      throw (new Exception(oErr.sMessage));
    }
  }

  protected void tearDown() throws Exception {
    m_oHarvestBehaviors = null;
    super.tearDown();
  }

  /**
   * This tests that a change of species is correctly handled in the harvest
   * behaviors.
   */
  public void testSpeciesChange() {
    GUIManager oManager = null;
        try {
          oManager = new GUIManager(null);
          oManager.ClearCurrentData();

          //Set up and parse our test file
          //Create our parser
          XMLReader oParser = XMLReaderFactory.createXMLReader(
              "org.apache.xerces.parsers.SAXParser");
          //Create the handler that looks for setup data and register it
          DefaultHandler oHandler = new ParameterFileParser(oManager);
          oParser.setContentHandler(oHandler);
          //Parse the file
          String sFileName = WriteXMLFile1();
          InputSource oToParse = new InputSource(sFileName);
          oParser.parse(oToParse);
          new java.io.File(sFileName).delete();

          TreePopulation oPop = oManager.GetTreePopulation();
          DisturbanceBehaviors oHarvest = oManager.GetDisturbanceBehaviors();

          //*************************
          //Verify the initial file read
          //*************************
          assertEquals(4, oHarvest.mp_oHarvestCuts.size());

          //First cut event:  only applied to Subalpine Fir
          HarvestData oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(0);
          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(0));

          //Second cut event:  applied to all nine species
          oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(1);
          assertEquals(9, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(3));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(4));
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(5));
          assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"), oCut.GetSpecies(6));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(7));
          assertEquals(oPop.GetSpeciesCodeFromName("Hybrid_Spruce"), oCut.GetSpecies(8));

          //Third cut event:  applied to four species
          oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(2);
          assertEquals(4, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(3));

          //Fourth cut event:  applied to Amabilis Fir
          oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(3);
          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));

          //Mortality episodes
          assertEquals(4, oHarvest.mp_oMortEpisodes.size());

          //Eposide one:  applied to Hybrid Spruce
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(0);
          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Hybrid_Spruce"), oCut.GetSpecies(0));

          //Episode two:  applied to all nine species
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(1);
          assertEquals(9, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(3));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(4));
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(5));
          assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"), oCut.GetSpecies(6));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(7));
          assertEquals(oPop.GetSpeciesCodeFromName("Hybrid_Spruce"), oCut.GetSpecies(8));

          //Episode three:  Applied to four species
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(2);
          assertEquals(4, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(3));

          //Episode four:  Applied to Amabilis Fir
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(3);
          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));

          //*********************************************
          //Now:  Remove Hybrid Spruce.
          //*********************************************
          String[] sSpecies = new String[] {"Western Hemlock",
             "Western Redcedar", "Amabilis Fir", "Subalpine Fir",
             "Lodgepole Pine", "Trembling Aspen", "Black Cottonwood",
             "Paper Birch"};

          oPop.SetSpeciesNames(sSpecies);

          assertEquals(4, oHarvest.mp_oHarvestCuts.size());

          //First cut event
          oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(0);
          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(0));

          //Second cut event
          oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(1);
          assertEquals(8, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(3));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(4));
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(5));
          assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"), oCut.GetSpecies(6));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(7));

          //Third cut event:  applied to four species
          oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(2);
          assertEquals(4, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(3));

          //Fourth cut event:  applied to Amabilis Fir
          oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(3);
          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));

          //Mortality episodes
          assertEquals(3, oHarvest.mp_oMortEpisodes.size());

          //Eposide one
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(0);
          assertEquals(8, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(3));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(4));
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(5));
          assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"), oCut.GetSpecies(6));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(7));

          //Episode two
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(1);
          assertEquals(4, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(3));

          //Episode three
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(2);
          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));

          //*********************************************
          //Now:  Remove Subalpine Fir and Amabilis Fir, and rearrange the
          //species names.
          //*********************************************
          sSpecies = new String[] {"Western Hemlock", "Black Cottonwood",
              "Western Redcedar", "Lodgepole Pine", "Trembling Aspen",
              "Paper Birch"};

        oPop.SetSpeciesNames(sSpecies);

        assertEquals(2, oHarvest.mp_oHarvestCuts.size());

        //First cut event
        oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(0);
        assertEquals(6, oCut.GetNumberOfSpecies());
        assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"), oCut.GetSpecies(0));
        assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"), oCut.GetSpecies(1));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(2));
        assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(3));
        assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"), oCut.GetSpecies(4));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(5));

        //Second cut event
        oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(1);
        assertEquals(3, oCut.GetNumberOfSpecies());
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(0));
        assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(1));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(2));

        //Mortality episodes
        assertEquals(2, oHarvest.mp_oMortEpisodes.size());

        //Eposide one
        oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(0);
        assertEquals(6, oCut.GetNumberOfSpecies());
        assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"), oCut.GetSpecies(0));
        assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"), oCut.GetSpecies(1));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(2));
        assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(3));
        assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"), oCut.GetSpecies(4));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(5));

        //Episode two
        oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(1);
        assertEquals(3, oCut.GetNumberOfSpecies());
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(0));
        assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(1));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(2));

        //*********************************************
          //Add a species - shouldn't change anything
          //*********************************************
        sSpecies = new String[] {"Western Hemlock", "Black Cottonwood",
              "Western Redcedar", "Lodgepole Pine", "Black Cherry",
              "Trembling Aspen", "Paper Birch"};

        oPop.SetSpeciesNames(sSpecies);

        assertEquals(2, oHarvest.mp_oHarvestCuts.size());

        //First cut event
        oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(0);
        assertEquals(6, oCut.GetNumberOfSpecies());
        assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"), oCut.GetSpecies(0));
        assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"), oCut.GetSpecies(1));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(2));
        assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(3));
        assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"), oCut.GetSpecies(4));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(5));

        //Second cut event
        oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(1);
        assertEquals(3, oCut.GetNumberOfSpecies());
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(0));
        assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(1));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(2));

        //Mortality episodes
        assertEquals(2, oHarvest.mp_oMortEpisodes.size());

        //Eposide one
        oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(0);
        assertEquals(6, oCut.GetNumberOfSpecies());
        assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"), oCut.GetSpecies(0));
        assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"), oCut.GetSpecies(1));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(2));
        assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(3));
        assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"), oCut.GetSpecies(4));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(5));

        //Episode two
        oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(1);
        assertEquals(3, oCut.GetNumberOfSpecies());
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(0));
        assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(1));
        assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(2));

        }
    catch (ModelException oErr) {
      fail("Disturbance behavior species change failed with message " + oErr.sMessage);
    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  public void testCompactHarvestList() throws ModelException {
    Vector oList = null;
    Vector expectedReturn = null;
    Vector actualReturn = m_oHarvestBehaviors.CompactHarvestList(oList);
    assertEquals("return value", expectedReturn, actualReturn);

    //--------------------------------------------------------------------
    //Case where all objects in the list can be compacted into one
    //--------------------------------------------------------------------
    HarvestData oData;
    Plot oPlot = null;
    oList = new Vector(0);
    //Normal processing

    try {
      oPlot = new Plot(null);
      oPlot.SetPlotXLength(100);
      oPlot.SetPlotYLength(100);

      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(1);
      oList.add(oData);

      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList.add(oData);

      actualReturn = m_oHarvestBehaviors.CompactHarvestList(oList);

      //Make sure there's only one object
      assertEquals(1, actualReturn.size());

      //Get the object and make sure it matches
      oData = (HarvestData) actualReturn.elementAt(0);
      assertEquals(1, oData.GetTimestep());
      assertEquals(oData.GetCutType(), DisturbanceBehaviors.CLEAR_CUT);
      assertEquals(oData.GetCutAmountType(),
                   DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      assertEquals(3, oData.GetNumberOfCells());
      assertEquals(3, oData.GetNumberOfSpecies());
      assertEquals(2, oData.GetNumberOfCutRanges());
      assertEquals(0, oData.GetSpecies(0));
      assertEquals(1, oData.GetSpecies(1));
      assertEquals(2, oData.GetSpecies(2));

      //--------------------------------------------------------------------
      //Case where there is no compaction
      //--------------------------------------------------------------------
      oList.clear();
      actualReturn.clear();

      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has different cut amount type
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has different cut type
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has different timestep
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(2);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has same cut range but additional
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCutRange(20, 30, 35);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has same cut ranges but one less
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has different cut range in low dbh
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(15, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has different cut range in high dbh
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 23, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has different cut range in cut amount
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 24);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has same cells but one extra
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(5, 5, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has same cells but one less
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has different cells in X
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(2, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //Next item has different cells in Y
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 15);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 4, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      actualReturn = m_oHarvestBehaviors.CompactHarvestList(oList);

      assertEquals(13, actualReturn.size());

      //--------------------------------------------------------------------
      //Case where there is some compaction
      //--------------------------------------------------------------------
      oList.clear();
      actualReturn.clear();

      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //This one is different
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(2);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList.add(oData);

      //This one should be added to the first
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(3);
      oList.add(oData);

      //This one should be added to the second
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(2);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList.add(oData);

      actualReturn = m_oHarvestBehaviors.CompactHarvestList(oList);

      assertEquals(2, actualReturn.size());

    }
    catch (ModelException oErr) {
      System.out.println(
          "Normal processing failed for list compacting with message " +
          oErr.sMessage);
    }
  }

  public void testAddHarvestData() {
    Vector oList = null;
    HarvestData oData = null;
    Plot oPlot;

    try {
      oPlot = new Plot(null);
      oPlot.SetPlotXLength(100);
      oPlot.SetPlotYLength(100);

      //Null list and harvest data
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(null, oList);

      //Harvest data with no cut ranges
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(1);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);

      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(null, oList);

      //First one to be added to a null list
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(1, oList.size());

      //Next item has different cut amount type
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(2, oList.size());

      //Next item has different cut type
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(3, oList.size());

      //Next item has different timestep
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(2);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(4, oList.size());

      //Next item has same cut range but additional
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 20);
      oData.AddCutRange(20, 30, 30);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(5, oList.size());

      //Next item has same cut ranges but one less
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(10, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(6, oList.size());

      //Next item has different cut range in low dbh
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(15, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(7, oList.size());

      //Next item has different cut range in high dbh
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 25, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(8, oList.size());

      //Next item has different cut range in cut amount
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(9, oList.size());

      //Next item has different species
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.CLEAR_CUT);
      oData.SetTimestep(2);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 20);
      oData.AddCell(0, 1, oPlot);
      oData.AddCell(3, 4, oPlot);
      oData.AddCell(6, 3, oPlot);
      oData.AddSpecies(0);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);


      //Those should have all been unique additions - test that
      assertEquals(10, oList.size());

      //This item is the same as the ninth one except for cells - should be
      //combined with it
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(5, 5, oPlot);
      oData.AddCell(7, 8, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(10, oList.size());
      oData = (HarvestData) oList.elementAt(8);
      assertEquals(5, oData.GetNumberOfCells());

      //Now try to add that same one again - nothing should change
      oData = null;
      oData = new HarvestData();
      oData.SetCutAmountType(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA);
      oData.SetCutType(DisturbanceBehaviors.PARTIAL_CUT);
      oData.SetTimestep(1);
      oData.AddCutRange(0, 10, 10);
      oData.AddCutRange(10, 20, 25);
      oData.AddCell(5, 5, oPlot);
      oData.AddCell(7, 8, oPlot);
      oData.AddSpecies(2);
      oList = m_oHarvestBehaviors.AddHarvestData(oList, oData);
      assertEquals(10, oList.size());
      oData = (HarvestData) oList.elementAt(8);
      assertEquals(5, oData.GetNumberOfCells());

    }
    catch (ModelException oErr) {
      System.out.println(
          "Normal processing failed for AddHarvestData with message " +
          oErr.sMessage);
    }
  }

  /**
   * Tests parameter file reading.
   */
  public void testReadXML() {
    GUIManager oManager = null;
        try {
          oManager = new GUIManager(null);
          oManager.ClearCurrentData();

          //Set up and parse our test file
          //Create our parser
          XMLReader oParser = XMLReaderFactory.createXMLReader(
              "org.apache.xerces.parsers.SAXParser");
          //Create the handler that looks for setup data and register it
          DefaultHandler oHandler = new ParameterFileParser(oManager);
          oParser.setContentHandler(oHandler);
          //Parse the file
          String sFileName = WriteXMLFile1();
          InputSource oToParse = new InputSource(sFileName);
          oParser.parse(oToParse);

          TreePopulation oPop = oManager.GetTreePopulation();

          DisturbanceBehaviors oHarvest = oManager.GetDisturbanceBehaviors();

          //*************************
          //Test each harvest cut
          //*************************
          assertEquals(4, oHarvest.mp_oHarvestCuts.size());

          //Number one
          HarvestData oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(0);
          assertEquals(1, oCut.GetTimestep());
          assertEquals(DisturbanceBehaviors.PARTIAL_CUT, oCut.GetCutType());
          assertEquals(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA, oCut.GetCutAmountType());
          assertEquals(4, oCut.GetNumberOfCutRanges());

          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(0));

          assertEquals(0.0, oCut.GetLowerBound(0), 0.001);
          assertEquals(30.0, oCut.GetUpperBound(0), 0.001);
          assertEquals(100.0, oCut.GetCutAmount(0), 0.001);

          assertEquals(30.0, oCut.GetLowerBound(1), 0.001);
          assertEquals(40.0, oCut.GetUpperBound(1), 0.001);
          assertEquals(70.0, oCut.GetCutAmount(1), 0.001);

          assertEquals(60.0, oCut.GetLowerBound(2), 0.001);
          assertEquals(70.0, oCut.GetUpperBound(2), 0.001);
          assertEquals(50.0, oCut.GetCutAmount(2), 0.001);

          assertEquals(45.0, oCut.GetLowerBound(3), 0.001);
          assertEquals(47.0, oCut.GetUpperBound(3), 0.001);
          assertEquals(20.0, oCut.GetCutAmount(3), 0.001);

          assertEquals(1, oCut.GetNumberOfCells());
          assertEquals(6, oCut.GetCell(0).GetX());
          assertEquals(7, oCut.GetCell(0).GetY());

          //Number two
          oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(1);
          assertEquals(2, oCut.GetTimestep());
          assertEquals(DisturbanceBehaviors.GAP_CUT, oCut.GetCutType());
          assertEquals(DisturbanceBehaviors.PERCENTAGE_DENSITY, oCut.GetCutAmountType());
          assertEquals(1, oCut.GetNumberOfCutRanges());

          assertEquals(9, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(3));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(4));
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(5));
          assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"), oCut.GetSpecies(6));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(7));
          assertEquals(oPop.GetSpeciesCodeFromName("Hybrid_Spruce"), oCut.GetSpecies(8));

          assertEquals(0.0, oCut.GetLowerBound(0), 0.001);
          assertEquals(3000.0, oCut.GetUpperBound(0), 0.001);
          assertEquals(100.0, oCut.GetCutAmount(0), 0.001);

          assertEquals(1, oCut.GetNumberOfCells());
          assertEquals(1, oCut.GetCell(0).GetX());
          assertEquals(2, oCut.GetCell(0).GetY());

          //Number three
          oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(2);
          assertEquals(3, oCut.GetTimestep());
          assertEquals(DisturbanceBehaviors.CLEAR_CUT, oCut.GetCutType());
          assertEquals(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA, oCut.GetCutAmountType());
          assertEquals(1, oCut.GetNumberOfCutRanges());

          assertEquals(4, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(3));

           assertEquals(20.0, oCut.GetLowerBound(0), 0.001);
          assertEquals(3000.0, oCut.GetUpperBound(0), 0.001);
          assertEquals(100.0, oCut.GetCutAmount(0), 0.001);

          assertEquals(19, oCut.GetNumberOfCells());
          assertEquals(8, oCut.GetCell(0).GetX());
          assertEquals(6, oCut.GetCell(0).GetY());

          assertEquals(8, oCut.GetCell(1).GetX());
          assertEquals(7, oCut.GetCell(1).GetY());

          assertEquals(8, oCut.GetCell(2).GetX());
          assertEquals(8, oCut.GetCell(2).GetY());

          assertEquals(8, oCut.GetCell(3).GetX());
          assertEquals(9, oCut.GetCell(3).GetY());

          assertEquals(8, oCut.GetCell(4).GetX());
          assertEquals(10, oCut.GetCell(4).GetY());

          assertEquals(8, oCut.GetCell(5).GetX());
          assertEquals(11, oCut.GetCell(5).GetY());

          assertEquals(8, oCut.GetCell(6).GetX());
          assertEquals(12, oCut.GetCell(6).GetY());

          assertEquals(8, oCut.GetCell(7).GetX());
          assertEquals(13, oCut.GetCell(7).GetY());

          assertEquals(8, oCut.GetCell(8).GetX());
          assertEquals(14, oCut.GetCell(8).GetY());

          assertEquals(8, oCut.GetCell(9).GetX());
          assertEquals(15, oCut.GetCell(9).GetY());

          assertEquals(8, oCut.GetCell(10).GetX());
          assertEquals(16, oCut.GetCell(10).GetY());

          assertEquals(8, oCut.GetCell(11).GetX());
          assertEquals(17, oCut.GetCell(11).GetY());

          assertEquals(8, oCut.GetCell(12).GetX());
          assertEquals(18, oCut.GetCell(12).GetY());

          assertEquals(8, oCut.GetCell(13).GetX());
          assertEquals(19, oCut.GetCell(13).GetY());

          assertEquals(8, oCut.GetCell(14).GetX());
          assertEquals(20, oCut.GetCell(14).GetY());

          assertEquals(8, oCut.GetCell(15).GetX());
          assertEquals(21, oCut.GetCell(15).GetY());

          assertEquals(8, oCut.GetCell(16).GetX());
          assertEquals(22, oCut.GetCell(16).GetY());

          assertEquals(8, oCut.GetCell(17).GetX());
          assertEquals(23, oCut.GetCell(17).GetY());

          assertEquals(8, oCut.GetCell(18).GetX());
          assertEquals(24, oCut.GetCell(18).GetY());

          //Number four
          oCut = (HarvestData) oHarvest.mp_oHarvestCuts.elementAt(3);
          assertEquals(4, oCut.GetTimestep());
          assertEquals(DisturbanceBehaviors.PARTIAL_CUT, oCut.GetCutType());
          assertEquals(DisturbanceBehaviors.ABSOLUTE_DENSITY, oCut.GetCutAmountType());
          assertEquals(1, oCut.GetNumberOfCutRanges());

          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));

          assertEquals(0.0, oCut.GetLowerBound(0), 0.001);
          assertEquals(3000.0, oCut.GetUpperBound(0), 0.001);
          assertEquals(100.0, oCut.GetCutAmount(0), 0.001);

          assertEquals(1, oCut.GetNumberOfCells());
          assertEquals(11, oCut.GetCell(0).GetX());
          assertEquals(21, oCut.GetCell(0).GetY());

          //*************************
          //Test each mortality episode
          //*************************
          assertEquals(4, oHarvest.mp_oMortEpisodes.size());

          //Number one
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(0);
          assertEquals(2, oCut.GetTimestep());
          assertEquals(DisturbanceBehaviors.PARTIAL_CUT, oCut.GetCutType());
          assertEquals(DisturbanceBehaviors.PERCENTAGE_BASAL_AREA, oCut.GetCutAmountType());
          assertEquals(4, oCut.GetNumberOfCutRanges());

          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Hybrid_Spruce"), oCut.GetSpecies(0));

          assertEquals(0.0, oCut.GetLowerBound(0), 0.001);
          assertEquals(31.0, oCut.GetUpperBound(0), 0.001);
          assertEquals(100.0, oCut.GetCutAmount(0), 0.001);

          assertEquals(31.0, oCut.GetLowerBound(1), 0.001);
          assertEquals(41.0, oCut.GetUpperBound(1), 0.001);
          assertEquals(71.0, oCut.GetCutAmount(1), 0.001);

          assertEquals(61.0, oCut.GetLowerBound(2), 0.001);
          assertEquals(71.0, oCut.GetUpperBound(2), 0.001);
          assertEquals(51.0, oCut.GetCutAmount(2), 0.001);

          assertEquals(46.0, oCut.GetLowerBound(3), 0.001);
          assertEquals(48.0, oCut.GetUpperBound(3), 0.001);
          assertEquals(21.0, oCut.GetCutAmount(3), 0.001);

          assertEquals(1, oCut.GetNumberOfCells());
          assertEquals(7, oCut.GetCell(0).GetX());
          assertEquals(8, oCut.GetCell(0).GetY());

          //Number two
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(1);
          assertEquals(3, oCut.GetTimestep());
          assertEquals(DisturbanceBehaviors.PARTIAL_CUT, oCut.GetCutType());
          assertEquals(DisturbanceBehaviors.PERCENTAGE_DENSITY, oCut.GetCutAmountType());
          assertEquals(1, oCut.GetNumberOfCutRanges());

          assertEquals(9, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Black_Cottonwood"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Lodgepole_Pine"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(3));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(4));
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(5));
          assertEquals(oPop.GetSpeciesCodeFromName("Paper_Birch"), oCut.GetSpecies(6));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(7));
          assertEquals(oPop.GetSpeciesCodeFromName("Hybrid_Spruce"), oCut.GetSpecies(8));

          assertEquals(1.0, oCut.GetLowerBound(0), 0.001);
          assertEquals(3000.0, oCut.GetUpperBound(0), 0.001);
          assertEquals(100.0, oCut.GetCutAmount(0), 0.001);

          assertEquals(1, oCut.GetNumberOfCells());
          assertEquals(2, oCut.GetCell(0).GetX());
          assertEquals(3, oCut.GetCell(0).GetY());

          //Number three
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(2);
          assertEquals(4, oCut.GetTimestep());
          assertEquals(DisturbanceBehaviors.PARTIAL_CUT, oCut.GetCutType());
          assertEquals(DisturbanceBehaviors.ABSOLUTE_BASAL_AREA, oCut.GetCutAmountType());
          assertEquals(1, oCut.GetNumberOfCutRanges());

          assertEquals(4, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Subalpine_Fir"), oCut.GetSpecies(0));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Redcedar"), oCut.GetSpecies(1));
          assertEquals(oPop.GetSpeciesCodeFromName("Trembling_Aspen"), oCut.GetSpecies(2));
          assertEquals(oPop.GetSpeciesCodeFromName("Western_Hemlock"), oCut.GetSpecies(3));

           assertEquals(21.0, oCut.GetLowerBound(0), 0.001);
          assertEquals(3000.0, oCut.GetUpperBound(0), 0.001);
          assertEquals(100.0, oCut.GetCutAmount(0), 0.001);

          assertEquals(19, oCut.GetNumberOfCells());
          assertEquals(9, oCut.GetCell(0).GetX());
          assertEquals(6, oCut.GetCell(0).GetY());

          assertEquals(9, oCut.GetCell(1).GetX());
          assertEquals(7, oCut.GetCell(1).GetY());

          assertEquals(9, oCut.GetCell(2).GetX());
          assertEquals(8, oCut.GetCell(2).GetY());

          assertEquals(9, oCut.GetCell(3).GetX());
          assertEquals(9, oCut.GetCell(3).GetY());

          assertEquals(9, oCut.GetCell(4).GetX());
          assertEquals(10, oCut.GetCell(4).GetY());

          assertEquals(9, oCut.GetCell(5).GetX());
          assertEquals(11, oCut.GetCell(5).GetY());

          assertEquals(9, oCut.GetCell(6).GetX());
          assertEquals(12, oCut.GetCell(6).GetY());

          assertEquals(9, oCut.GetCell(7).GetX());
          assertEquals(13, oCut.GetCell(7).GetY());

          assertEquals(9, oCut.GetCell(8).GetX());
          assertEquals(14, oCut.GetCell(8).GetY());

          assertEquals(9, oCut.GetCell(9).GetX());
          assertEquals(15, oCut.GetCell(9).GetY());

          assertEquals(9, oCut.GetCell(10).GetX());
          assertEquals(16, oCut.GetCell(10).GetY());

          assertEquals(9, oCut.GetCell(11).GetX());
          assertEquals(17, oCut.GetCell(11).GetY());

          assertEquals(9, oCut.GetCell(12).GetX());
          assertEquals(18, oCut.GetCell(12).GetY());

          assertEquals(9, oCut.GetCell(13).GetX());
          assertEquals(19, oCut.GetCell(13).GetY());

          assertEquals(9, oCut.GetCell(14).GetX());
          assertEquals(20, oCut.GetCell(14).GetY());

          assertEquals(9, oCut.GetCell(15).GetX());
          assertEquals(21, oCut.GetCell(15).GetY());

          assertEquals(9, oCut.GetCell(16).GetX());
          assertEquals(22, oCut.GetCell(16).GetY());

          assertEquals(9, oCut.GetCell(17).GetX());
          assertEquals(23, oCut.GetCell(17).GetY());

          assertEquals(9, oCut.GetCell(18).GetX());
          assertEquals(24, oCut.GetCell(18).GetY());

          //Number four
          oCut = (HarvestData) oHarvest.mp_oMortEpisodes.elementAt(3);
          assertEquals(5, oCut.GetTimestep());
          assertEquals(DisturbanceBehaviors.PARTIAL_CUT, oCut.GetCutType());
          assertEquals(DisturbanceBehaviors.ABSOLUTE_DENSITY, oCut.GetCutAmountType());
          assertEquals(1, oCut.GetNumberOfCutRanges());

          assertEquals(1, oCut.GetNumberOfSpecies());
          assertEquals(oPop.GetSpeciesCodeFromName("Amabilis_Fir"), oCut.GetSpecies(0));

          assertEquals(0.0, oCut.GetLowerBound(0), 0.001);
          assertEquals(3000.0, oCut.GetUpperBound(0), 0.001);
          assertEquals(100.0, oCut.GetCutAmount(0), 0.001);

          assertEquals(1, oCut.GetNumberOfCells());
          assertEquals(12, oCut.GetCell(0).GetX());
          assertEquals(22, oCut.GetCell(0).GetY());

          new java.io.File(sFileName).delete();
        }
    catch (ModelException oErr) {
      fail("XML tree map reading failed with message " + oErr.sMessage);
    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }


  }

  /**
   * This writes an XML file to test value setting.
   * @return Name of file.
   * @throws ModelException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected String WriteXMLFile1() throws ModelException {
    try {
      String sFileName = "\\loratestxml1.xml";
      java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

      oOut.write(
          "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
      oOut.write("<paramFile fileCode=\"06010101\">");

      //Plot
      oOut.write("<plot>");
      oOut.write("<plot_lenX>200.0</plot_lenX>");
      oOut.write("<plot_lenY>200.0</plot_lenY>");
      oOut.write("<plot_latitude>55.37</plot_latitude>");
      oOut.write("</plot>");

      oOut.write("<trees>");
      oOut.write("<tr_speciesList>");
      oOut.write("<tr_species speciesName=\"Western_Hemlock\"/>");
      oOut.write("<tr_species speciesName=\"Western_Redcedar\"/>");
      oOut.write("<tr_species speciesName=\"Amabilis_Fir\"/>");
      oOut.write("<tr_species speciesName=\"Subalpine_Fir\"/>");
      oOut.write("<tr_species speciesName=\"Hybrid_Spruce\"/>");
      oOut.write("<tr_species speciesName=\"Lodgepole_Pine\"/>");
      oOut.write("<tr_species speciesName=\"Trembling_Aspen\"/>");
      oOut.write("<tr_species speciesName=\"Black_Cottonwood\"/>");
      oOut.write("<tr_species speciesName=\"Paper_Birch\"/>");
      oOut.write("</tr_speciesList>");
      oOut.write("</trees>");

      //Include an empty behavior list - this triggers setup
      oOut.write("<behaviorList>");
      oOut.write("</behaviorList>");

      oOut.write("<harvest>");

      //Cut event:  cut type of partial, cut amount percent of basal area,
      //four cut ranges.  First cut range tags go low, hi, amt, second cut
      //range tags go low, amt, hi, third cut range tags go amt, lo, hi.
      //One species.
      oOut.write("<ha_cutEvent>");
      oOut.write("<ha_applyToSpecies species=\"Subalpine_Fir\" />");
      oOut.write("<ha_timestep>1</ha_timestep>");
      oOut.write("<ha_cutType>partial</ha_cutType>");
      oOut.write("<ha_cutAmountType>percent of basal area</ha_cutAmountType>");
      oOut.write("<ha_dbhRange>");
      oOut.write("<ha_low>0.0</ha_low>");
      oOut.write("<ha_high>30.0</ha_high>");
      oOut.write("<ha_amountToCut>100.0</ha_amountToCut>");
      oOut.write("</ha_dbhRange>");
      oOut.write("<ha_dbhRange>");
      oOut.write("<ha_low>30.0</ha_low>");
      oOut.write("<ha_amountToCut>70.0</ha_amountToCut>");
      oOut.write("<ha_high>40.0</ha_high>");
      oOut.write("</ha_dbhRange>");
      oOut.write("<ha_dbhRange>");
      oOut.write("<ha_amountToCut>50.0</ha_amountToCut>");
      oOut.write("<ha_low>60.0</ha_low>");
      oOut.write("<ha_high>70.0</ha_high>");
      oOut.write("</ha_dbhRange>");
      oOut.write("<ha_dbhRange>");
      oOut.write("<ha_low>45.0</ha_low>");
      oOut.write("<ha_high>47.0</ha_high>");
      oOut.write("<ha_amountToCut>20.0</ha_amountToCut>");
      oOut.write("</ha_dbhRange>");
      oOut.write("<ha_applyToCell x=\"6\" y=\"7\" />");
      oOut.write("</ha_cutEvent>");

      //Cut event:  cut type of gap, cut amount percent of density.  All
      //species.  One cell.
      oOut.write("<ha_cutEvent>");
      oOut.write("<ha_applyToSpecies species=\"Amabilis_Fir\" /> ");
      oOut.write("<ha_applyToSpecies species=\"Black_Cottonwood\" /> ");
      oOut.write("<ha_applyToSpecies species=\"Lodgepole_Pine\" /> ");
      oOut.write("<ha_applyToSpecies species=\"Western_Redcedar\" />");
      oOut.write("<ha_applyToSpecies species=\"Trembling_Aspen\" />");
      oOut.write("<ha_applyToSpecies species=\"Subalpine_Fir\" />");
      oOut.write("<ha_applyToSpecies species=\"Paper_Birch\" />");
      oOut.write("<ha_applyToSpecies species=\"Western_Hemlock\" />");
      oOut.write("<ha_applyToSpecies species=\"Hybrid_Spruce\" />");
      oOut.write("<ha_timestep>2</ha_timestep>");
      oOut.write("<ha_cutType>gap</ha_cutType>");
      oOut.write("<ha_cutAmountType>percent of density</ha_cutAmountType>");
      oOut.write("<ha_dbhRange>");
      oOut.write("<ha_low>0.0</ha_low>");
      oOut.write("<ha_high>3000.0</ha_high>");
      oOut.write("<ha_amountToCut>100.0</ha_amountToCut>");
      oOut.write("</ha_dbhRange>");
      oOut.write("<ha_applyToCell x=\"1\" y=\"2\" />");
      oOut.write("</ha_cutEvent>");

      //Cut event:  cut type of clear, cut amount absolute basal area.  Some
      //species.  Many cells.
      oOut.write("<ha_cutEvent>");
      oOut.write("<ha_applyToSpecies species=\"Subalpine_Fir\" />");
      oOut.write("<ha_applyToSpecies species=\"Western_Redcedar\" />");
      oOut.write("<ha_applyToSpecies species=\"Trembling_Aspen\" />");
      oOut.write("<ha_applyToSpecies species=\"Western_Hemlock\" />");
      oOut.write("<ha_timestep>3</ha_timestep>");
      oOut.write("<ha_cutType>clear</ha_cutType>");
      oOut.write("<ha_cutAmountType>absolute basal area</ha_cutAmountType>");
      oOut.write("<ha_dbhRange>");
      oOut.write("<ha_low>20.0</ha_low>");
      oOut.write("<ha_high>3000.0</ha_high>");
      oOut.write("<ha_amountToCut>100.0</ha_amountToCut>");
      oOut.write("</ha_dbhRange>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"6\"/> ");
      oOut.write("<ha_applyToCell x=\"8\" y=\"7\"/> ");
      oOut.write("<ha_applyToCell x=\"8\" y=\"8\"/> ");
      oOut.write("<ha_applyToCell x=\"8\" y=\"9\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"10\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"11\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"12\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"13\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"14\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"15\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"16\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"17\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"18\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"19\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"20\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"21\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"22\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"23\"/>");
      oOut.write("<ha_applyToCell x=\"8\" y=\"24\"/>");
      oOut.write("</ha_cutEvent>");


      //Cut event:  cut amount absolute density.
      oOut.write("<ha_cutEvent>");
      oOut.write("<ha_applyToSpecies species=\"Amabilis_Fir\" />");
      oOut.write("<ha_timestep>4</ha_timestep>");
      oOut.write("<ha_cutType>partial</ha_cutType>");
      oOut.write("<ha_cutAmountType>absolute density</ha_cutAmountType>");
      oOut.write("<ha_dbhRange>");
      oOut.write("<ha_low>0.0</ha_low>");
      oOut.write("<ha_high>3000.0</ha_high>");
      oOut.write("<ha_amountToCut>100.0</ha_amountToCut>");
      oOut.write("</ha_dbhRange>");
      oOut.write("<ha_applyToCell x=\"11\" y=\"21\" />");
      oOut.write("</ha_cutEvent>");
      oOut.write("</harvest>");

      //**********************************
      // Mortality episode
      //**********************************
      oOut.write("<disturbance>");

      //Mortality episode:  cut amount percent of basal area,
      //four cut ranges.  First cut range tags go low, hi, amt, second cut
      //range tags go low, amt, hi, third cut range tags go amt, lo, hi.
      //One species.
      oOut.write("<ds_deathEvent>");
      oOut.write("<ds_applyToSpecies species=\"Hybrid_Spruce\" />");
      oOut.write("<ds_timestep>2</ds_timestep>");
      oOut.write("<ds_cutAmountType>percent of basal area</ds_cutAmountType>");
      oOut.write("<ds_dbhRange>");
      oOut.write("<ds_low>0.0</ds_low>");
      oOut.write("<ds_high>31.0</ds_high>");
      oOut.write("<ds_amountToCut>100.0</ds_amountToCut>");
      oOut.write("</ds_dbhRange>");
      oOut.write("<ds_dbhRange>");
      oOut.write("<ds_low>31.0</ds_low>");
      oOut.write("<ds_amountToCut>71.0</ds_amountToCut>");
      oOut.write("<ds_high>41.0</ds_high>");
      oOut.write("</ds_dbhRange>");
      oOut.write("<ds_dbhRange>");
      oOut.write("<ds_amountToCut>51.0</ds_amountToCut>");
      oOut.write("<ds_low>61.0</ds_low>");
      oOut.write("<ds_high>71.0</ds_high>");
      oOut.write("</ds_dbhRange>");
      oOut.write("<ds_dbhRange>");
      oOut.write("<ds_low>46.0</ds_low>");
      oOut.write("<ds_high>48.0</ds_high>");
      oOut.write("<ds_amountToCut>21.0</ds_amountToCut>");
      oOut.write("</ds_dbhRange>");
      oOut.write("<ds_applyToCell x=\"7\" y=\"8\" />");
      oOut.write("</ds_deathEvent>");

      //Mortality episode:  cut amount percent of density.  All
      //species.  One cell.
      oOut.write("<ds_deathEvent>");
      oOut.write("<ds_applyToSpecies species=\"Amabilis_Fir\" />");
      oOut.write("<ds_applyToSpecies species=\"Black_Cottonwood\" />");
      oOut.write("<ds_applyToSpecies species=\"Lodgepole_Pine\" />");
      oOut.write("<ds_applyToSpecies species=\"Western_Redcedar\" />");
      oOut.write("<ds_applyToSpecies species=\"Trembling_Aspen\" />");
      oOut.write("<ds_applyToSpecies species=\"Subalpine_Fir\" />");
      oOut.write("<ds_applyToSpecies species=\"Paper_Birch\" />");
      oOut.write("<ds_applyToSpecies species=\"Western_Hemlock\" />");
      oOut.write("<ds_applyToSpecies species=\"Hybrid_Spruce\" />");
      oOut.write("<ds_timestep>3</ds_timestep>");
      oOut.write("<ds_cutAmountType>percent of density</ds_cutAmountType>");
      oOut.write("<ds_dbhRange>");
      oOut.write("<ds_low>1.0</ds_low>");
      oOut.write("<ds_high>3000.0</ds_high>");
      oOut.write("<ds_amountToCut>100.0</ds_amountToCut>");
      oOut.write("</ds_dbhRange>");
      oOut.write("<ds_applyToCell x=\"2\" y=\"3\" />");
      oOut.write("</ds_deathEvent>");

      //Mortality episode:  cut amount absolute basal area.  Some
      //species.  Many cells.
      oOut.write("<ds_deathEvent>");
      oOut.write("<ds_applyToSpecies species=\"Subalpine_Fir\" />");
      oOut.write("<ds_applyToSpecies species=\"Western_Redcedar\" />");
      oOut.write("<ds_applyToSpecies species=\"Trembling_Aspen\" />");
      oOut.write("<ds_applyToSpecies species=\"Western_Hemlock\" />");
      oOut.write("<ds_timestep>4</ds_timestep>");
      oOut.write("<ds_cutAmountType>absolute basal area</ds_cutAmountType>");
      oOut.write("<ds_dbhRange>");
      oOut.write("<ds_low>21.0</ds_low>");
      oOut.write("<ds_high>3000.0</ds_high>");
      oOut.write("<ds_amountToCut>100.0</ds_amountToCut>");
      oOut.write("</ds_dbhRange>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"6\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"7\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"8\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"9\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"10\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"11\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"12\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"13\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"14\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"15\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"16\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"17\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"18\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"19\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"20\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"21\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"22\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"23\"/>");
      oOut.write("<ds_applyToCell x=\"9\" y=\"24\"/>");
      oOut.write("</ds_deathEvent>");


      //Mortality episode:  cut amount absolute density.
      oOut.write("<ds_deathEvent>");
      oOut.write("<ds_applyToSpecies species=\"Amabilis_Fir\" />");
      oOut.write("<ds_timestep>5</ds_timestep>");
      oOut.write("<ds_cutAmountType>absolute density</ds_cutAmountType>");
      oOut.write("<ds_dbhRange>");
      oOut.write("<ds_low>0.0</ds_low>");
      oOut.write("<ds_high>3000.0</ds_high>");
      oOut.write("<ds_amountToCut>100.0</ds_amountToCut>");
      oOut.write("</ds_dbhRange>");
      oOut.write("<ds_applyToCell x=\"12\" y=\"22\" />");
      oOut.write("</ds_deathEvent>");
      oOut.write("</disturbance>");

      oOut.write("</paramFile>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      ModelException oErr = new ModelException();
      oErr.sMessage = oE.getMessage();
      throw (oErr);
    }
  }
}
